#include "SDL.h"
#include "math.h"
#include <string>
#include <sstream>
#include <time.h>

#include "ball.h"
#include "constants.h"
#include "globals.h"
#include "functions.h"
#include "timer.h"


Ball::Ball(int w, int h) {
	Ball::w = w;
	Ball::h = h;
	
	srand(time(0));
	int t = rand() % 2;
	if (t == 0) {
		Ball::angle = (rand() % 90) + 135;
	} else {
		Ball::angle = (rand() % 90) + 315;
	}
	
	Ball::speed = 2;

	Ball::x = SCR_WIDTH / 2.f;
	Ball::y = SCR_HEIGHT / 2.f;

	Ball::startDelay = 1;
}

void Ball::restart() {
	Ball::x = SCR_WIDTH / 2.f;
	Ball::y = SCR_HEIGHT / 2.f;
	Ball::speed = 2;

	int t = rand() % 2;
	if (t == 0) {
		Ball::angle = (rand() % 90) + 135;
	} else {
		Ball::angle = (rand() % 90) + 315;
	}

	Ball::startDelay = SDL_GetTicks();

	SDL_FillRect(screen, NULL, 0x000000);
}

void Ball::update(Uint8* keystates) {
	// Temporary SDL_Rect for various operations
	SDL_Rect tRect;

	// Let the ball bounce against the top and bottom border
	if (Ball::y + (Ball::h / 2) > SCR_HEIGHT && Ball::bounceBot) {
		Ball::angle = 360 - Ball::angle;
	}

	if (Ball::y - (Ball::h / 2) < 40 && !Ball::bounceBot) {
		Ball::angle = 360 - Ball::angle;
	}
	
	// Make sure the angle is always in range of 360
	Ball::angle = (int)Ball::angle % 360;
	if (Ball::angle < 0) {
		Ball::angle = abs(-360 - Ball::angle);
	}
	
	// Detect which border it should bounce against, top or bottom
	if (Ball::angle > 180) {
		Ball::bounceBot = false;
	} else {
		Ball::bounceBot = true;
	}
	
	// Detect which player's bat it should bounce against the left, or the right bat
	if (Ball::angle > 90 && Ball::angle < 270) {
		Ball::bounceTarget = BALL_BOUNCE_P1;
	} else {
		Ball::bounceTarget = BALL_BOUNCE_P2;
	}

	// Delete the "Go!" Printed on top of the screen after 5 secs
	if (Ball::goTimer.get_ticks() <= 5000 && Ball::goTimer.get_ticks() > 0) {
		tSurface = TTF_RenderText_Solid(defaultFont, "Go!", clrGreenSDL);
		apply_surface(tSurface, screen, (SCR_WIDTH / 2) - (tSurface->w / 2), 15 - (tSurface->h / 2));
	} else {
		Ball::goTimer.reset_timer();
	}
	
	// If the game has started already, just do the normal process
	// Otherwise start a countdown, and release the ball at 3
	if (Ball::startDelay == 0) {
		// Check for collision with one of the bats, and if there is, mirrors the ball's angle
		SDL_Rect ballRect;
		SDL_Rect batRect;

		ballRect.x = Ball::x - Ball::w / 2;
		ballRect.y = Ball::y - Ball::h / 2;
		ballRect.w = Ball::w;
		ballRect.h = Ball::h;

		if (Ball::bounceTarget == BALL_BOUNCE_P1) {
			batRect.x = p1Bat.x_pos();
			batRect.y = p1Bat.y_pos();
			batRect.w = BAT_WIDTH;
			batRect.h = BAT_HEIGHT;
		} else {
			batRect.x = p2Bat.x_pos();
			batRect.y = p2Bat.y_pos();
			batRect.w = BAT_WIDTH;
			batRect.h = BAT_HEIGHT;
		}

		if (rect_intersect(&ballRect, &batRect)) {
			Ball::angle = calc_angle(&ballRect, &batRect) + 180;
			Ball::speed *= 1.1;
		}
		
		// Mechanic to steer the ball >:D
		if (keystates[SDLK_LEFT]) {
			Ball::angle -= 10;
		}
	
		if (keystates[SDLK_RIGHT]) {
			Ball::angle += 10;
		}
		
		// Calulate and apply the ball's new position
		float changeX = Ball::speed * cos(Ball::angle * DEG_TO_RAD);
		float changeY = Ball::speed * sin(Ball::angle * DEG_TO_RAD);

		Ball::x += changeX;
		Ball::y += changeY;

		if (Ball::x - (BALL_WIDTH / 2) < 0) {
			Ball::x = BALL_WIDTH / 2;
		}
		if (Ball::x + (BALL_WIDTH / 2) > SCR_WIDTH) {
			Ball::x = SCR_WIDTH - (BALL_WIDTH / 2);
		}
		
		// Draw the ball
		SDL_FillRect(screen, &ballRect, 0x00FF00);
	} else {
		int countdown = (SDL_GetTicks() - Ball::startDelay) / 1000;
		countdown = 3 - countdown;
		
		if (countdown <= 0) {
			Ball::startDelay = 0;
			std::stringstream textCountdown;

			textCountdown << countdown;
			tSurface = TTF_RenderText_Solid(scoreFont, "3", clrGreenSDL);

			SDL_Rect tRect;
			tRect.x = (SCR_WIDTH / 2) - (tSurface->w / 2);
			tRect.y = 15 - (tSurface->h / 2);
			tRect.w	= tSurface->w;
			tRect.h = tSurface->h;
			SDL_FillRect(screen, &tRect, 0x000000);

			tSurface = TTF_RenderText_Solid(defaultFont, "Go!", clrGreenSDL);
			apply_surface(tSurface, screen, (SCR_WIDTH / 2) - (tSurface->w / 2), 15 - (tSurface->h / 2));

			Ball::goTimer.start_timer();
		}
		
		if (Ball::startDelay > 0) {
			std::stringstream textCountdown;
			textCountdown << countdown;
			tSurface = TTF_RenderText_Solid(defaultFont, textCountdown.str().c_str(), clrGreenSDL);

			tRect.x = (SCR_WIDTH / 2) - (tSurface->w / 2);
			tRect.y = 15 - (tSurface->h / 2);
			tRect.w	= tSurface->w;
			tRect.h = tSurface->h;
			SDL_FillRect(screen, &tRect, 0x000000);

			apply_surface(tSurface, screen, (SCR_WIDTH / 2) - (tSurface->w / 2), 15 - (tSurface->h / 2));
		}


	}
}

int Ball::x_pos() {
	return (int)Ball::x;	
}

int Ball::y_pos() {
	return (int)Ball::y;
}

float Ball::give_angle() {
	return Ball::angle;
}

float Ball::give_speed() {
	return Ball::speed;
}

bool Ball::give_bounce_target() {
	return Ball::bounceTarget;
}