#include "SDL.h"
#include "math.h"

#include "bat.h"
#include "constants.h"
#include "globals.h"

Bat::Bat(int x, int y, int w, int h, int speed, int upKey, int downKey) {
	Bat::x = x;
	Bat::y = y;
	Bat::w = w;
	Bat::h = h;
	Bat::speed = speed;
	Bat::upKey = upKey;
	Bat::downKey = downKey;
	Bat::aiEnabled = false;
}

void Bat::bat_move(int modifier) {
	Bat::y += Bat::speed * modifier;

	if (Bat::y < 40) {Bat::y = 40;}
	if (Bat::y + Bat::h > SCR_HEIGHT) {Bat::y = SCR_HEIGHT - Bat::h;}
}

void Bat::bat_update(Uint8* keystates, Uint32 clr) {
	oldX = Bat::x;
	oldY = Bat::y;
	
	// Bat AI Routine
	if (Bat::aiEnabled) {
		// Determine which Bat you're dealing with: the left or the right one
		if (Bat::x > SCR_WIDTH / 2) {
			// If the ball is flying to the other bat
			if (ball.give_bounce_target() == BALL_BOUNCE_P1) {
				if (Bat::y + (BAT_HEIGHT / 4) > SCR_HEIGHT / 2) {
					Bat::bat_move(-1);
				} else if (Bat::y + ((BAT_HEIGHT / 4) * 3) < SCR_HEIGHT / 2){
					Bat::bat_move(1);
				}
			// If the ball is flying to this bat
			} else {
				if (Bat::y + (BAT_HEIGHT / 4) > ball.y_pos()) {
					Bat::bat_move(-1);
				} else if (Bat::y + ((BAT_HEIGHT / 4) * 3) < ball.y_pos()) {
					Bat::bat_move(1);
				}
			}
		} else {
			// If the ball is flying to the other bat
			if (ball.give_bounce_target() == BALL_BOUNCE_P2) {
				if (Bat::y + (BAT_HEIGHT / 4) > SCR_HEIGHT / 2) {
					Bat::bat_move(-1);
				} else if (Bat::y + ((BAT_HEIGHT / 4) * 3) < SCR_HEIGHT / 2) {
					Bat::bat_move(1);
				}
			// if the ball is flying to this bat
			} else {
				if (Bat::y + (BAT_HEIGHT / 4) > ball.y_pos()) {
					Bat::bat_move(-1);
				} else if (Bat::y + ((BAT_HEIGHT / 4) * 3) < ball.y_pos()) {
					Bat::bat_move(1);
				}
			}
		}
	// If AI Isn't enabled
	} else {
		if (keystates[Bat::upKey]) {
			Bat::bat_move(-1);
		}
		if (keystates[Bat::downKey]) {
			Bat::bat_move(1);
		}
	}

	SDL_Rect tRect;
	tRect.h = Bat::h;
	tRect.w = Bat::w;
	tRect.x = Bat::oldX;
	tRect.y = Bat::oldY;
	
	SDL_FillRect(screen, &tRect, clrBlack);

	tRect.x = Bat::x;
	tRect.y = Bat::y;

	SDL_FillRect(screen, &tRect, clr);
}

void Bat::bat_speed(int newSpeed) {
	Bat::speed = newSpeed;
}

void Bat::bat_resize(int w, int h) {
	int fixedY = Bat::y + ceil(Bat::h * .5);
	int newY = fixedY - ceil(h * .5);

	int fixedX = Bat::x + ceil(Bat::w * .5);
	int newX = fixedX - ceil(w * .5);

	Bat::x = newX;
	Bat::y = newY;
	Bat::w = w;
	Bat::h = h;
	
	// To make sure the bat doesn't go outside the window
	bat_move(0);
}

void Bat::set_ai(bool newStatus) {
	Bat::aiEnabled = newStatus;
}

int Bat::x_pos() {
	return Bat::x;
}

int Bat::y_pos() {
	return Bat::y;
}

bool Bat::ai_enabled() {
	return Bat::aiEnabled;
}